/*
 * Quasseldroid - Quassel client for Android
 *
 * Copyright (c) 2024 Janne Mareike Koschinski
 * Copyright (c) 2024 The Quassel Project
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import io.github.gradlenexus.publishplugin.NexusPublishExtension
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.publish.PublishingExtension
import org.gradle.api.publish.maven.MavenPublication
import org.gradle.kotlin.dsl.configure
import org.gradle.kotlin.dsl.get
import org.gradle.kotlin.dsl.getByType
import org.gradle.kotlin.dsl.repositories
import org.gradle.plugins.signing.SigningExtension

class PublicationConvention : Plugin<Project> {
  override fun apply(target: Project) {
    with(target) {
      with(pluginManager) {
        apply("maven-publish")
        apply("signing")
      }

      version = rootProject.version
      group = rootProject.group

      val canSign = project.properties.keys
        .any { it.startsWith("signing.") }

      configure<PublishingExtension> {
        publications {
          create("mavenJava", MavenPublication::class.java) {
            val projectName = project.name
              .removePrefix("core")
              .removePrefix("-")
            artifactId = buildArtifactName(
              extractArtifactGroup(project.group as String),
              rootProject.name,
              projectName.ifEmpty { null }
            )
            from(components["java"])

            pom {
              name.set(buildHumanReadableName(artifactId))
              description.set("Pure-Kotlin implementation of the Quassel Protocol")
              url.set("https://git.kuschku.de/justJanne/libquassel")
              licenses {
                license {
                  name.set("Mozilla Public License Version 2.0")
                  url.set("https://www.mozilla.org/en-US/MPL/2.0/")
                }
              }
              developers {
                developer {
                  id.set("justJanne")
                  name.set("Janne Mareike Koschinski")
                }
              }
              scm {
                connection.set("scm:git:https://git.kuschku.de/justJanne/libquassel.git")
                developerConnection.set("scm:git:ssh://git.kuschku.de:2222/justJanne/libquassel.git")
                url.set("https://git.kuschku.de/justJanne/libquassel")
              }
            }
          }
        }
      }

      if (canSign) {
        configure<SigningExtension> {
          sign(extensions.getByType<PublishingExtension>().publications["maven"])
        }

        configure<NexusPublishExtension> {
          repositories {
            sonatype()
          }
        }
      }
    }
  }

  private fun buildArtifactName(group: String? = null, project: String? = null, module: String? = null): String {
    return removeConsecutive(listOfNotNull(group, project, module).flatMap { it.split('-') })
      .joinToString("-")
  }

  private fun buildHumanReadableName(name: String) = name
    .splitToSequence('-')
    .joinToString(" ", transform = String::capitalize)

  private fun extractArtifactGroup(group: String): String? {
    // split into parts by domain separator
    val elements = group.split('.')
    // drop the tld/domain part, e.g. io.datalbry
    val withoutDomain = elements.drop(2)
    // if anything remains, that’s our artifact group
    return withoutDomain.lastOrNull()
  }

  private fun <T> removeConsecutive(list: List<T>): List<T> {
    val result = mutableListOf<T>()
    for (el in list) {
      if (el != result.lastOrNull()) {
        result.add(el)
      }
    }
    return result
  }
}
